function [csout]=compose(bin)
%
% Funcio que troba la minima composite structure d'una estructura
% binaria.
%
% Autor: Josep Maria Mirats i Tur
% Data: 13-VI-1999, Barcelona
%
% Parmetres:
%
%	Entrada: bin, matriu de relacions binries
%
%	Sortida: csout, composite structure
%
% Us de la funci:
%
%		[csout]=compose(bin)
%
echo off
%
% Ordena la matriu de relacions binaries
%
bin = sortrows(bin);
%
% Elimina les relacions binries repetides
%
bin = elim_repet(bin);
[row,col]=size(bin);
%
% Forma matriu amb informaci dels possibles subconjunts
%
nvar = max([max(bin(:,1)),max(bin(:,2))]);
matriu = zeros(nvar-1,nvar);
for i=1:row,
 a = bin(i,1);
 b = bin(i,2);
 if ( a>0 & b>0 )
  matriu(a,b) = 1;
 end
end
%
% Forma el vector que cont els possibles subconjunts
%
cst = [];
for i=1:nvar-1,
 vec = sort([i,find(matriu(i,:))]);
 [a,b] = size(vec);
 for j = 2:b,
  aux = nchoosek(vec,j);
  for k = 1:size(aux,1),
   cst = [cst 0 aux(k,:)];
  end
 end
end
[a,b] = size(cst);
cst = cst(2:b);
%
% Comprova el cas especial en que b es 0
%
if ( b == 0 )
 k = 1;
 csout = zeros(1,(nvar*2)-1);
 for i=1:nvar,
  csout(k) = i;
  k = k+2;
 end
 return
end
%
% Post-procs del vector cst
%
% Elimina subsistemes repetits
%
cst = elim_sub_rep(cst);
%
% Elimina subsistemes amb relacions binries 'no originals'
%
csout = elim_norig(cst,bin);
%
% Elimina subsistemes inclosos en altres
%
csout = elim_fil_incloses(csout);
%
% Elimina subsistemes tal que les seves relacions binries estan
% els altres subsistemes
%
csout = elim_subs_bin(csout);
%
% Comprova si falten variables i les afegeix si cal
%
[a,b] = size(csout);
for i = 1:nvar,
 j = size(find(csout == i),2);
 if ( j == 0 )
  csout = [csout,0,i];
 end
end
return
