function [bout,pout,csout] = fcombine(b1,p1,b2,p2,stri)
%
% Funci que recombina dos matrius de comportament en una
%
% Autor: Josep Maria Mirats i Tur
% Data: 5-II-1999, Barcelona
%
% Parmetres:
%
%	Entrada: b1, behavior del primer subsistema
%	   	 p1, vector de confiances del primer subsistema
%		 b2, behavior del segn subsistema
%	   	 p2, vector de confiances del segn subsistema
%		 stri, vector que cont els dos subsistemes en forma de
%			composite structure
%
%	Sortida: bout, behavior del sistema recombinat
%		 pout, vector de confiances del sistema recombinat
%		 csout, estructura resultant
%
%	Us de la funci:
%
%			[bout,pout] = fcombine(b1,p1,b2,p2,stri)
%
%
bout=[];
pout=[];
%
% Troba les dues estructures i comprova que n'hi ha dues
%
aux = find(stri == 0);
if ( length(aux) ~= 1 )
 fprintf('\n FCOMBINE. Sn necessries dues estructures \n\n')
 return
end
str1 = stri(1:aux-1);
str2 = stri(aux+1:size(stri,2));
%
% Troba l'estructura resultant
%
[i,j,v] = find(stri);
csout = sort(v);
csout = elim_repet(csout');
csout = csout';
%
% Comprova les variables que sn iguals a les dues estructures. Posa punters
% de les variables de la primera estructura a les de la segona.
%
m1 = length(str1);
m2 = length(str2);
a = 0;
punter = zeros(1,m1);
for i = 1:m1,
 for j = 1:m2,
  if ( str1(i) == str2(j) )
   punter(i) = j;
   a = a+1;
  end
 end
end
%
% Troba de cada matriu de comportament els estats corresponents a les
% variables comunes. Aquests estats s'emmagatzemen a la matriu maux.
% En aquest tros de codi se suposa que els dos subsistemes a recombinar
% venen d'una operaci prvia d'extracci de les seves matrius de 
% comportament d'una matriu de comportament d'un sistema global. Amb 
% aquesta suposici noms cal buscar els estats de les variables comunes
% a b1 o b2 ja que en teoria seran els mateixos. Si els dos subsistemes
% no vingussin d'una operaci prvia d'extracci podria ser que les
% variables comunes d'un i altre subsistema tingussin estats diferents.
% Aquest cas no s contempla aqu.
%
%
n1 = size(b1,1);
n2 = size(b2,1);
k = 0;
% 
% Recombina les estructures
% Primer cas, si no hi ha cap variable igual en les dues estructures
%
if ( a == 0 )
 for i=1:n1,
  for j=1:n2,
   k = k+1;
   for l = 1:m1,
    aux = str1(l);
    bout(k,aux) = b1(i,l);
   end
   for l = 1:m2,
    aux = str2(l);
    bout(k,aux) = b2(j,l);
   end
   pout(k) = min([p1(i) p2(j)]);
  end
 end
%
% Segon cas. Quan alguna de les variables coincideix
%
else
 for i = 1:n1,
  vaux = zeros(1,n2);
  flag = 0; p = 0;
  for j = 1:n2,
   match = 1;
   for l=1:m1,
    aux = punter(l);
    if ( aux ~= 0 )
     if ( b2(j,aux) ~= b1(i,l) )
      match = 0;
     end
    end
   end
   if ( match == 1 )
    vaux(j) = 1;
%    p = p+p2(j); 
    flag = 1;
   end
  end
  for j = 1:n2,
   if ( vaux(j) == 1 )
    k = k+1;
    for l = 1:m1,
     aux = str1(l);
     bout(k,aux) = b1(i,l);
    end
    for l = 1:m2,
     aux = str2(l);
     bout(k,aux) = b2(j,l);
    end
    pout(k) = min([p1(i) p2(j)]);
   end
  end
 end
end
%
pout=pout';
%
% Agafa les columnes correctes de bout i ordena-les
%
a = size(csout,2);
for i = 1:a,
 bout(:,i) = bout(:,csout(i));
end
bout = bout(:,1:a);
aux = [bout pout];
aux = sortrows(aux,[1:a]);
bout = aux(:,1:a);
pout = aux(:,a+1);
%
