function [csout] = singleref(beh,prob,emax,fid)
%
% Implementa algorisme de SINGLEREF
%
% Parametres d'entrada:
%
%       beh - Matriu behavior del sistema
%       prob - Vector de probabilitat del sistema
%       emax - Error maxim permes per eliminar una relacio binaria
%       fid - Indica si volem fitxer amb resultats intermitjos
%
% Inicialitza variables
%
[mb,nb] = size(beh);
%
%
% Troba les relacions binaries del sistema
% i escriu capalera al fitxer
%
clear cst;
for i=1:nb,
  cst(i) = i;
end;
bin = binary(cst);
if fid ~= 0
  fprintf(fid,'ESTADISTICA DE RELACIONS BINARIES\n\n\n');
  fprintf(fid,'\tERROR\t\tRELACIO BINARIA OMESA\n\n');
end;
[nbin,b] = size(bin);
errst = zeros(nbin,1);
%
% Bucle que elimina una relacio binaria i comproba quin es l'error
% de reconstruccio sense aquesta relacio binaria. Si l'error es mes
% gran o igual que EMAX la guarda.
%
binout = zeros(1,2);	% Matriu de relacions binaries resultants
for i=1:nbin,
  binaux = [bin(1:i-1,:);bin(i+1:nbin,:)];
  clear cst;
  cst = compose(binaux);
  aux = max(cst);              % Proteccio necessaria per a que no
  if aux ~= nb                % falli la funcio structure en cas
    for j=1:nb-aux,           % de que en les relacions binaries
      cst = [cst,0,aux+j];      % de binaux no estiguin els nombres
    end;                      % representant a totes les variables
  end;
  errst(i) = structure(beh,prob,cst,1);
  if errst(i) >= emax binout = [binout;bin(i,:)]; end;
  if fid ~= 0 
    fprintf(fid,'\t%6.6f\t\t%d , %d\n',errst(i),bin(i,1),bin(i,2)); 
  end;
end;
%
[a,b] = size(binout);
binout = binout(2:a,:);
%
% Trova estructura resultant i escriu les ultimes linies al fitxer 
%
if ( a>1 )
  csout = compose(binout);
  aux = max(csout);             % Proteccio necessaria per escriure
  if aux ~= nb                  % correctament al fitxer en cas
    for j=1:nb-aux,             % de que en les relacions binaries
      csout = [csout,0,aux+j];  % de binout no estiguin els nombres
    end;                        % representant a totes les variables
  end;
else
  for i=1:nb*2,
    csout(i) = 0;
  end;
  j = 0;
  for i=1:nb,
    csout(i+j) = i;
    j = i;
  end;
  csout = csout(1:(nb*2)-1);
end;
%
if fid ~= 0
  fprintf(fid,'\nES CONSIDEREN LES RELACIONS AMB ERROR MAJOR DE %6.4f',emax);
  fprintf(fid,'\n\n\nESTRUCTURA RESULTANT: (');
  [a,b]=size(csout);
  cs2 = [csout,0];
  for i=1:b,
    if cs2(i) ~= 0 fprintf(fid,'%d',cs2(i)); end;
    if ( cs2(i+1) ~= 0 & cs2(i) ~= 0 ) fprintf(fid,','); end;
    if cs2(i) == 0 fprintf(fid,') ('); end;
  end;
  fprintf(fid,')\n');
end;
return
