function [err] = structure(b,p,stri,alg)
%
% Funci que calcula l'error de reconstruir b1 i p1 a partir dels subsistemes
% que hi ha a stri
%
% Autor: Josep Maria Mirats i Tur
% Data: 4-II-1999, Barcelona
%
% Parmetres:
%
%       Entrada: b, behavior del sistema
%                p, vector de probabilitats del sistema
%                stri, vector que cont els subsistemes en forma de
%                       composite structure
%		 alg, frmula a usar per calcular l'error
%
%			alg = 1, err = sum(abs(p_recons-p))
%
%			alg = 2, err = sqr(sum((p_reconsi-pi)^2)) 
%				Frmula de Klir
%
%       Sortida: err, error de reconstrucci del sistema recombinat
%
%       Us de la funci:
%
%                       [err] = structure(b,p,stri,alg)
%
%
err = 0;
%
% Comprova que la behavior i el vector de prob. d'entrada sn compatibles
%
a = size(b,1);
c = size(p,1);
if ( a ~= c )
 fprintf('\n STRUCTURE. Behavior vector de probabilitats no compatibles \n\n')
 return
end
%
% Troba el nmero de subsistemes
%
index = [0 find(stri == 0) size(stri,2)+1];
n = length(index)-1;
if ( n < 2 )
 fprintf('\n STRUCTURE. Com a mnim sn necessris dos subsistems \n\n')
 return
end
%
% Extreu el comportament del primer subsistema
%
str1 = stri(index(1)+1:index(2)-1);
[b1,p1] = extract(b,p,str1);
%
% Bucle que combina i extreu subsistemes fins a completar la reconstrucci
%
for i = 2:n,
 str2 = stri(index(i)+1:index(i+1)-1);
 [b2,p2] = extract(b,p,str2);
 [b1,p1,str1] = combine(b1,p1,b2,p2,[str1 0 str2]);
end
%
% Si en el sistema reconstrut hi ha nous estats, completa el vector de 
% probabilitats del sistema original
%
n = size(b1,1);
m = size(b,1);
mm = m;
if ( n > m )
 j = 0;
 k = 0;
 for i = 1:n,
  if ( j == mm ) j = mm-1; end
  if ( b(j+1,:) == b1(i,:) )
   j = j+1;
   k = k+1;
  else
   p = [p(1:k); 0 ; p(k+1:m)];
   k = k+1;
   m = m+1;
  end
 end
end
%
% Calcula l'error de la reconstrucci
%
if ( alg == 1)
 err = 0.5*sum(abs(p1-p));
end
if (alg == 2 )
 err = sqrt((p1-p)'*(p1-p));
end
%
